@@ -1,8 +1,11 @@
-from typing import Callable, Any, Optional, List
+from typing import Callable, Any, List
 
 import networkx as nx
 
-from .types import Variable, Function, GraphDict
+from .types import Variable, Function, Data, GraphDict
+from .draw import draw_compute_graph
+from .dynamic import split_static_dynamic
+from .utils import expand_nested_dict, get_with_injected_parameters, get_input_variables
 
 """
 Graph building functions
@@ -20,6 +23,8 @@ def build_dag(graph_dict: GraphDict, local_source_name="graph_locals") -> nx.DiG
     """
     dag = nx.digraph.DiGraph()
     for name, node_spec in graph_dict.items():
+        if not (isinstance(node_spec, Function) or isinstance(node_spec, Data)):
+            raise TypeError("Invalid node type, expected Function or Data", node_spec)
         dag.add_node(name, node_spec=node_spec)
     trace_edges(dag, local_source_name)
     return dag
@@ -68,7 +73,13 @@ Operations over built DiGraphs
 """
 
 
-def build_callable(dag: nx.DiGraph, local_source_name="graph_locals") -> Callable[[dict], Any]:
+def build_callable(
+    dag: nx.DiGraph,
+    local_source_name="graph_locals",
+    nested_params=True,
+    include_inputs=False,
+    out_keys=None,
+) -> Callable[[dict], Any]:
     """Returns a callable Python function corresponding to this graph
 
     Args:
@@ -79,49 +90,46 @@ def build_callable(dag: nx.DiGraph, local_source_name="graph_locals") -> Callabl
         The corresponding Python function for this graph
     """
 
+    ggen = nx.topological_sort(dag)
+
+    node_dict = {node: dag.nodes[node]["node_spec"] for node in ggen}
+
     def compute_from_params(**kwargs):
         out_p = {}
-        ggen = nx.topological_sort(dag)
-        sources = kwargs
-        for node in ggen:
-            node_spec = dag.nodes[node]["node_spec"]
+
+        if nested_params:
+            sources = {}
+            for k, v in kwargs.items():
+                sources[k] = expand_nested_dict(v, include_parents=True)
+        else:
+            sources = kwargs.copy()
+
+        for node, node_spec in node_dict.items():  # ggen:
+            # node_spec = dag.nodes[node]["node_spec"]
             if isinstance(node_spec, Variable):
-                out_p[node] = kwargs[node_spec.source][node_spec.name]
+                out_p[node] = sources[node_spec.source][node_spec.name]
             elif isinstance(node_spec, Function):
                 sources.update({local_source_name: out_p})
                 out_p[node] = node_spec.call(sources)
+            elif isinstance(node_spec, Data):
+                out_p[node] = node_spec.data
             else:
                 raise Exception("Unsupported node type", node, node_spec, type(node_spec))
+
+        if include_inputs:
+            out_p.update({k: v for k, v in sources.items() if k != "graph_locals"})
+
         return out_p
 
-    return compute_from_params
+    if out_keys is None:
+        return compute_from_params
+    else:
 
+        def compute_for_keys(**kwargs):
+            results = compute_from_params(**kwargs)
+            return {k: results[k] for k in out_keys}
 
-def draw_graph(dag: nx.DiGraph, targets: Optional[List[str]] = None):
-    """A basic plotting function for viewing the data flow in a graph
-
-    Args:
-        dag: A computegraph DiGraph
-        targets: A list of output nodes to mark with a separate colour
-    """
-    pos = nx.nx_agraph.graphviz_layout(dag, prog="dot")
-    node_specs = nx.get_node_attributes(dag, "node_spec")
-
-    labels = {k: k for k in dag}
-
-    if targets is None:
-        targets = []
-
-    def get_color(name, param):
-        if isinstance(param, Variable):
-            return "lightgreen"
-        elif name in targets:
-            return "#ee88ee"
-        else:
-            return "lightblue"
-
-    node_colors = [get_color(name, param) for name, param in node_specs.items()]
-    return nx.draw(dag, pos=pos, labels=labels, node_color=node_colors, width=0.4, node_size=500)
+        return compute_for_keys
 
 
 """
@@ -140,9 +148,21 @@ class ComputeGraph:
         """
         self.dag = build_dag(graph_dict, local_source_name)
         self.local_source_name = local_source_name
+        self.pdag = get_with_injected_parameters(self.dag)
+        self.dict = graph_dict
 
-    def draw(self, targets=None):
-        return draw_graph(self.dag, targets)
+    def draw(self, targets=None, **kwargs):
+        return draw_compute_graph(self.pdag, targets, **kwargs)
 
-    def get_callable(self) -> Callable[[dict], Any]:
-        return build_callable(self.dag, self.local_source_name)
+    def freeze(self, dynamic_inputs: List[Variable], targets: List[str], inputs: dict):
+        return split_static_dynamic(self.dict, dynamic_inputs, targets, **inputs)
+
+    def get_input_variables(self):
+        return get_input_variables(self.dag)
+
+    def get_callable(
+        self, nested_params=True, include_inputs=False, out_keys=None
+    ) -> Callable[[dict], Any]:
+        return build_callable(
+            self.dag, self.local_source_name, nested_params, include_inputs, out_keys
+        )
