@@ -27,6 +27,14 @@ class Variable:
         return (self.name == other.name) and (self.source == other.source)
 
 
+class Data:
+    def __init__(self, data):
+        self.data = data
+
+    def __repr__(self):
+        return f"Data: {str(self.data)}"
+
+
 def local(name: str) -> Variable:
     """Convenience function for returning a graph_locals variable
 
@@ -39,6 +47,18 @@ def local(name: str) -> Variable:
     return Variable(name, source="graph_locals")
 
 
+def param(name: str) -> Variable:
+    """Convenience function for returning a parameters variable
+
+    Args:
+        name: Variable Name
+
+    Returns:
+        Variable with source "parameters"
+    """
+    return Variable(name, source="parameters")
+
+
 class Function:
     """Universal wrapper for callable functions, whose args and kwargs
     may be either Variables (resolved at run-time), or any other Python value
@@ -62,6 +82,9 @@ class Function:
     def __repr__(self):
         return f"Function: func={self.func}, args={self.args}, kwargs={self.kwargs})"
 
+    def __eq__(self, other):
+        return self.func == other.func and self.args == other.args and self.kwargs == other.kwargs
+
     def build_args(self, sources: dict) -> Tuple[tuple, dict]:
         from .utils import build_args
 
